@@ -1,12 +1,14 @@
 package DBIx::ModelUpdate;
 
-use 5.006;
+use 5.005;
 
 require Exporter;
 
-our $VERSION = '0.3';
+our $VERSION = '0.61';
 
 use Data::Dumper;
+use Storable    ('freeze', 'dclone');
+use Digest::MD5 'md5_base64';
 
 no strict;
 no warnings;
@@ -54,12 +56,12 @@ sub new {
 }
 
 ################################################################################
-
-sub unquote_table_name {
-	my ($name) = @_;
-	$name =~ s{\W}{}g;
-	return lc $name;
-}
+#
+#sub unquote_table_name {
+#	my ($name) = @_;
+#	$name =~ s{\W}{}g;
+#	return lc $name;
+#}
 
 ################################################################################
 
@@ -72,7 +74,7 @@ sub get_tables {
 			$_ => { columns => $self -> get_columns ($_, $options), keys => $self -> get_keys ($_) } 
 		} 
 		map {
-			unquote_table_name ($_)
+			$self -> unquote_table_name ($_)
 		} 
 		($self -> {db} -> tables)
 	};
@@ -85,15 +87,41 @@ sub assert {
 
 	my ($self, %params) = @_;
 	
+	$Storable::canonical = 1;
+	
+	my $serial = freeze (\%params);
+
+#	my $serial = Dumper (\%params);
+	
+#	my $checksum = md5_base64 (freeze (\%params));
+	
+	my $checksum = md5_base64 ($serial);
+	
 	my $needed_tables = $params {tables};
 	
 	my $existing_tables = $self -> get_tables;	
 	
+	unless (exists $existing_tables -> {'_db_model_checksums'}) {
+	
+		$self -> do ('CREATE TABLE _db_model_checksums (checksum CHAR(22))');
+		$self -> do ('CREATE INDEX _db_model_checksums_pk ON _db_model_checksums (checksum)');
+	
+	} else {
+	
+		my $st = $self -> {db} -> prepare ('SELECT COUNT(*) FROM _db_model_checksums WHERE checksum = ?');
+		$st -> execute ($checksum);
+		my ($cnt) = $st -> fetchrow_array;
+		$st -> finish;
+		
+		return if $cnt;
+	
+	}
+	
 	while (my ($name, $definition) = each %$needed_tables) {
 	
 		while (my ($dc_name, $dc_definition) = each %{$params {default_columns}}) {
 			
-			$definition -> {columns} -> {$dc_name} ||= $dc_definition;
+			$definition -> {columns} -> {$dc_name} ||= dclone ($dc_definition);
 			
 		};
 
@@ -154,6 +182,12 @@ sub assert {
 		map { $self -> insert_or_update ($name, $_, $definition) } @{$definition -> {data}} if $definition -> {data};
 		
 	}
+	
+	$serial =~ s{\'}{\\\'}g; #'
+	
+	$self -> do ("INSERT INTO _db_model_checksums (checksum) VALUES ('$checksum')");
+	
+#print STDERR Dumper (\%params);
 			
 }
 
